#include "main.h"
#include "suart.h"



#define BIT_TIME  (u8)((XTAL / BAUD) / PRESCALE)


volatile u8 stx_count;
u8 stx_data;

volatile u8 srx_done;
u8 srx_data;
u8 srx_mask;
u8 srx_tmp;


void suart_init( void )
{
	SBIT(PORTD,STX) = 1;
	// OCR0A = TCNT0 + 1;		// force first compare
	// TCCR0A = 0;				// Default: Normal port operation
	TCCR0B = 1 << CS01 | 1<<CS00;	// clk/64
	// TIFR0 = 1<<ICF1;			// clear pending interrupt
	TIMSK0 = 1 << OCIE0A;		// enable output compare interrupt

	EICRA = 1 << ISC01;			// falling edge
	EIMSK = 1 << INT0;			// enable edge interrupt

	stx_count = 0;				// nothing to sent
	srx_done = 0;		        // nothing received
	STXDDR |= 1 << STX;			// TX output
}


u8 sgetchar (void)      // get byte
{
  while (!srx_done);      // wait until byte received
  srx_done = 0;
  return srx_data;
}


ISR (INT0_vect)    // rx start
{
	OCR0B = TCNT0 + (u8)((BIT_TIME * 3) / 2);// scan 1.5 bits after start

	srx_tmp = 0;        // clear bit storage
	srx_mask = 1;        // bit mask
	if( !(SRXPIN & 1<<SRX))  {  // still low
		EIMSK &= ~(1 << INT0);			// disable edge interrupt
		TIMSK0 = 1<<OCIE0A^1<<OCIE0B;  // wait for first bit
}
	TIFR0 = 1<<OCF0B;      // clear pending interrupt ? why does that output compare int occur?
EIFR |= (1 << INTF0);		// clear any pending edge interrupt
}


ISR (TIMER0_COMPB_vect)
{
	u8 in = SRXPIN;      // scan rx line
  // PORTB = 0x00;
	if (srx_mask) {
    	if (in & 1 << SRX)
			srx_tmp |= srx_mask;
		srx_mask <<= 1;
		OCR0B += BIT_TIME;      // next bit slice
	} else {
		srx_done = 1;      // mark rx data valid
		srx_data = srx_tmp;      // store rx data
		// TIFR0 = 1<<ICF1;      // clear pending interrupt
		TIMSK0 = 1<<OCIE0A;        // enable tx and wait for start
		EIFR |= (1 << INTF0);		// clear any pending edge interrupt: This hinders the in0-vect from beeing triggerd again just now which may occur by falling edges in the serial data bits
		EIMSK = 1 << INT0;	// reenable edge interrupt
	}
	  // PORTB = 0x02;
}


void sputchar (u8 val)      // send byte
{
  while (stx_count);      // until last byte finished
  stx_data = ~val;      // invert data for Stop bit generation
  stx_count = 10;      // 10 bits: Start + data + Stop
}


void sputs (u8 *txt)      // send string
{
  while (*txt)
    sputchar (*txt++);
}


ISR (TIMER0_COMPA_vect)    // tx bit
{
  u8 dout;
  u8 count;

  OCR0A += BIT_TIME;      // next bit slice
  count = stx_count;

	if (count) {
		stx_count = --count;    // count down
		// ??? dout = 1<<COM0A1;      // set low on next compare
		dout = 0;
		if (count != 9) {      // no start bit
			if (!(stx_data & 1))    // test inverted data
				//??? dout = 1<<COM0A1^1<<COM0A0;  // set high on next compare
				dout = 1;
      		stx_data >>= 1;      // shift zero in from left
		}
		// TCCR0A = dout;
		// STX = (dout == 0) ? 1 : 0;
		SBIT(PORTD,STX) = dout;
	}
}
